package com.jjt.biz.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.jjt.biz.vo.KnittingEmpVO;
import com.jjt.biz.vo.SalaryVO;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.emp.domain.TwinEmpCalc;
import com.jjt.emp.service.ITwinEmpCalcService;
import com.jjt.wkEmp.domain.TwinWkEmpSync;
import com.jjt.wkEmp.service.ITwinWkEmpRotaService;
import com.jjt.wkEmp.service.ITwinWkEmpSyncService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 第三方数据接口
 *
 * @author wukai
 */
@Api(tags = "第三方数据接口")
@RestController
@Slf4j
public class ApiThirdController extends BaseController {
    @Resource
    ITwinWkEmpSyncService syncService;
    @Resource
    ITwinWkEmpRotaService rotaService;
    @Resource
    private ITwinEmpCalcService twinEmpCalcService;

    @ApiOperation("经编车间员工排班")
    @PostMapping("/api/knitting/emp/schedule")
    @CrossOrigin(origins = "*")
    public AjaxResult empSchedule(@RequestBody KnittingEmpVO vo) {
        TwinWkEmpSync sync = new TwinWkEmpSync();
        Date s = new Date();
        sync.setRecordTime(s);
        sync.setStatus("N");
        ObjectMapper objectMapper = new ObjectMapper();
        try {
            String jsonString = objectMapper.writeValueAsString(vo);
            sync.setContent(jsonString);
            syncService.insertTwinWkEmpSync(sync);
            rotaService.sync(vo);
            Date e = new Date();
            long time = e.getTime() - s.getTime();
            sync.setTime(time);
            sync.setStatus("Y");
            syncService.updateTwinWkEmpSync(sync);
        } catch (Exception e) {
            e.printStackTrace();
            AjaxResult.error(e.getMessage());
        }
        return AjaxResult.success();
    }

    @ApiOperation("经编车间员工月度工资")
    @GetMapping("/api/salary/monthly")
    @CrossOrigin(origins = "*")
    public AjaxResult monthCalc(String date) {
        List<TwinEmpCalc> list = twinEmpCalcService.selectTwinEmpCalcListByMonth(date);
        Map<String, String> nameMap = getNameMapping();
        if (list.size() > 0) {
            List<SalaryVO> salary = list.stream()
                    .collect(Collectors.groupingBy(TwinEmpCalc::getEmpName))
                    .entrySet().stream()
                    .map(entry -> {
                                List<TwinEmpCalc> group = entry.getValue();
                                BigDecimal totalPrice = calculateTotalPrice(group);
                                SalaryVO vo = new SalaryVO();
                                String id = nameMap.get(entry.getKey());
                                if (id == null) {
                                    id = "";
                                }
                                vo.setEmpId(id);
                                vo.setEmpName(entry.getKey());
                                vo.setSalary(totalPrice.setScale(2, RoundingMode.HALF_UP));
                                return vo;
                            }
                    ).collect(Collectors.toList());
            return AjaxResult.success(salary);
        } else {
            return AjaxResult.error("无此月份数据");
        }
    }

    /**
     * 辅助方法：计算总金额
     */
    private BigDecimal calculateTotalPrice(List<TwinEmpCalc> group) {
        return group.stream()
                .map(TwinEmpCalc::getTotalPrice).filter(Objects::nonNull)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
    }


    public Map<String, String> getNameMapping() {
        Map<String, String> nameMap = new LinkedHashMap<>();
        nameMap.put("杨玉叶", "20130902001");
        nameMap.put("裴菊芬", "20140219001");
        nameMap.put("黄春叶", "20181102001");
        nameMap.put("马雯鑫", "20200131002");
        nameMap.put("余茂敏", "20200831003");
        nameMap.put("李红子", "20200921002");
        nameMap.put("姚虹云", "20210305003");
        nameMap.put("马爱敏", "20211011003");
        nameMap.put("吴亚琼", "20211101005");
        nameMap.put("马雨欣", "20220217003");
        nameMap.put("谭裕琴", "20220315006");
        nameMap.put("张蓉", "20220922001");
        nameMap.put("邓威", "20230511003");
        nameMap.put("杨艳杰", "20230908005");
        nameMap.put("沈让", "20231109005");
        nameMap.put("邓丽", "20231127003");
        nameMap.put("史杨杨", "20231204003");
        nameMap.put("史利侠", "20240228012");
        nameMap.put("黄浩杰", "20240626006");
        nameMap.put("费秋容", "20250208005");
        nameMap.put("韩忍", "20250215002");
        nameMap.put("袁奇俊", "20250217008");
        nameMap.put("孙秋来", "20250701004");
        nameMap.put("伍小弟", "20250704002");
        return nameMap;
    }

}
